#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "stereo.h"

/* calcul de la moyenne d'une image par fenetre glissante */
void
calculMoyenne(const float *imagedata, int dimx, int dimy,
	      int windowsize, float *moyennedata)
{
    /* demi-taille de la fenetre */
    const int halfwindow = (windowsize - 1) / 2;
    /* tableaux de la taille d'une ligne, contenant la somme d'une colonne
       de la hauteur de la fenetre, et le nombre de points de cette colonne */
    double *somme_colonne;
    int *cardinal_colonne;
    /* somme et cardinal sur la fenetre glissante */
    double somme_fenetre;
    int cardinal_fenetre;
    /* position du centre de la fenetre */
    int x, y;
    /* position du dernier pixel en bas a droite de la fenetre */
    /* on a toujours
       x_corner == x+halfwindow
       y_corner == y+halfwindow */
    int x_corner, y_corner;
    /* variable d'optimisation */
    int ydimx;

    /* verification des entrees */
    assert((windowsize > 0) && ((windowsize % 2) == 1));

    /* allocation */
    somme_colonne = (double *) malloc(dimx * sizeof(double));
    cardinal_colonne = (int *) malloc(dimx * sizeof(int));
    /* on met tout a zero */
    memset(somme_colonne, 0, dimx * sizeof(double));
    memset(cardinal_colonne, 0, dimx * sizeof(int));

    y_corner = -1;
    /* go! */
    for (y = 0; y < dimy; y++) {
	/* y_corner doit etre egal a y+halfwindow... */
	while (y_corner != (y + halfwindow)) {
	    y_corner++;
	    /* soustraire la ligne perdue, qui est en y_corner-windowsize,
	       si elle existe */
	    if ((y_corner - windowsize) >= 0) {
		ydimx = (y_corner - windowsize) * dimx;
		for (x = 0; x < dimx; x++) {
		    cardinal_colonne[x]--;
		    somme_colonne[x] -= imagedata[x + ydimx];
		}
	    }
	    /* ajouter la ligne y_corner aux stats, si elle existe */
	    if (y_corner < dimy) {
		ydimx = y_corner * dimx;
		for (x = 0; x < dimx; x++) {
		    cardinal_colonne[x]++;
		    somme_colonne[x] += imagedata[x + ydimx];
		}
	    }
	}
	/* OK, a ce point-la somme_colonne et cardinal_colonne sont a jour */
	/* reste a faire glisser la fenetre sur l'axe x */
	somme_fenetre = 0.0;
	cardinal_fenetre = 0;
	x_corner = -1;
	for (x = 0; x < dimx; x++) {
	    /* x_corner doit etre egal a x+halfwindow... */
	    while (x_corner != (x + halfwindow)) {
		x_corner++;
		/* soustraire la colonne perdue, qui est en x_corner-windowsize,
		   si elle existe */
		if ((x_corner - windowsize) >= 0) {
		    cardinal_fenetre -=
			cardinal_colonne[x_corner - windowsize];
		    somme_fenetre -= somme_colonne[x_corner - windowsize];
		}
		/* ajouter la colonne x_corner aux stats, si elle existe */
		if (x_corner < dimx) {
		    cardinal_fenetre += cardinal_colonne[x_corner];
		    somme_fenetre += somme_colonne[x_corner];
		}
	    }
	    /* OK, a ce point-la somme_fenetre et cardinal_fenetre sont a jour */
	    moyennedata[x + y * dimx] = somme_fenetre / cardinal_fenetre;
	}
    }
    free(somme_colonne);
    free(cardinal_colonne);
}
