#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>
#include "pm_util.h"

#include "stereo.h"

/* #define ZERO_MEAN */
#define COURBE_SCORE

void
image_gray_to_float(const gray * input, int dimx, int dimy, float *output)
{
    int xy;
    const int max_xy = dimx * dimy;

    for (xy = 0; xy < max_xy; xy++) {
	output[xy] = input[xy];
    }
}

void
image_float_to_gray(const float * input, int dimx, int dimy, gray *output,
		    float *minp, float *maxp)
{
    int xy;
    const int max_xy = dimx * dimy;
    float min, max, val;

    min = *minp;
    max = *maxp;
    if ((min == 0.) && (max == 0)) {
	min = max = input[0];
	for (xy = 1; xy < max_xy; xy++) {
	    if (input[xy] < min)
		min = input[xy];
	    else if (input[xy] > max)
		max = input[xy];
	}
	*minp = min;
	*maxp = max;
    }

    for (xy = 0; xy < max_xy; xy++) {
	val = 255. * (input[xy] - min) / (max - min);
	if(val < 0)
	    output[xy] = 0;
	else if(val > 255)
	    output[xy] = 255;
	else
	    output[xy] = val;
    }
}

int main(int argc, char **argv)
{
    gray *image1, *image2, *dispg, *scoreg;
    float *image1f, *image2f, *dispf, *dispbf, *scoref;
    float *moyenne1, *moyenne2;
    const char *image1name;
    const char *image2name;
    int dispmin, dispmax;
    int windowsize;
    int cols1, rows1, cols2, rows2;
    int maxval;
    float floatmin, floatmax;

    if (argc != 6) {
	fprintf(stderr, "Usage: %s <image1> <image2> <dispmin> <dispmax> <window_size>\n\
Description: Stereoscopie par correlation, calcul de l'image de disparite\n\
d(x,y) telle que le correspondant de image1(x,y) est image2(x+x(d,y)).\n\
windowsize doit etre impair.\n\
sortie dans disp.pgm (disparite) et score.pgm (score).\n",
		argv[0]);
	exit(1);
    }

    /* recuperation des arguments */
    image1name = argv[1];
    image2name = argv[2];
    dispmin = atoi(argv[3]);
    dispmax = atoi(argv[4]);
    windowsize = atoi(argv[5]);
    fprintf(stderr,
	    "image1=%s image2=%s dispmin=%d dispmax=%d windowsize=%d\n",
	    image1name, image2name, dispmin, dispmax, windowsize);
    if ((windowsize % 2) != 1) {
	fprintf(stderr, "windowsize n'est pas impair!\n");
	exit(1);
    }

    /* chargement des images, conversion en float */
    image1 = LoadPGMImage(argv[1], &cols1, &rows1, &maxval);
    image2 = LoadPGMImage(argv[2], &cols2, &rows2, &maxval);
    if ((rows1 != rows2) || (cols1 != cols2)) {
	fprintf(stderr, "Erreur : images de tailles differentes !\n");
	exit(1);
    }

    image1f = (float *) malloc(cols1 * rows1 * sizeof(float));
    image_gray_to_float(image1, cols1, rows1, image1f);
    free(image1);
    image2f = (float *) malloc(cols2 * rows2 * sizeof(float));
    image_gray_to_float(image2, cols2, rows2, image2f);
    free(image2);

#ifdef ZERO_MEAN
    /* calcul de la moyenne de l'image sur la fenetre */
    /* allocation d'une image de meme taille et meme type */
    moyenne1 = (float *) malloc(cols1 * rows1 * sizeof(float));
    /* la fonction calculMoyenne est dans moyenne.c */
    calculMoyenne(image1f, cols1, rows1, windowsize, moyenne1);
    /* soustraire moyenne a image, resultat dans image */
    soustraireImage(image1f, cols1, rows1, moyenne1, image1f);
    free(moyenne1);
    /* calcul de la moyenne de l'image sur la fenetre */
    /* allocation d'une image de meme taille et meme type */
    moyenne2 = (float *) malloc(cols2 * rows2 * sizeof(float));
    /* la fonction calculMoyenne est dans moyenne.c */
    calculMoyenne(image2f, cols2, rows2, windowsize, moyenne2);
    /* soustraire moyenne a image, resultat dans image */
    soustraireImage(image2f, cols2, rows2, moyenne2, image2f);
    free(moyenne2);
#endif

#ifdef COURBE_SCORE
    {
	int d;
	double score;
	const int x = 211, y = 159;
	/*const int x = 188, y = 244;*/ /* d(SSD)=-49.0602 d(ZSSD)=-49.6889 */
	for (d = dispmin; d <= dispmax; d++) {
	    score =
		scorePoint(image1f, cols1, rows1, image2f, d, windowsize,
			   x, y);
	    if (score != -1.0) {
		printf("%d %g\n", d, score);
	    }
	}
    }
#else
    dispf = (float *) malloc(cols1 * rows1 * sizeof(float));
    scoref = (float *) malloc(cols1 * rows1 * sizeof(float));
    fprintf(stderr,"Correlation aller...\n");
    correlationImages(image1f, cols1, rows1, image2f, dispmin, dispmax,
		      windowsize, dispf, scoref);
    fprintf(stderr,"Correlation retour...\n");
    /* A ECRIRE
    dispbf = (float *) malloc(cols1 * rows1 * sizeof(float));
    correlationImages(...);
    fprintf(stderr,"Validation...\n");
    validationAllerRetour(dispf, cols1, rows1, dispbf);
    free(dispbf);
    */
    dispg = (gray *) malloc(cols1 * rows1 * sizeof(gray));
    scoreg = (gray *) malloc(cols1 * rows1 * sizeof(gray));
    floatmin = floatmax = 0.0;
    image_float_to_gray(dispf, cols1, rows1, dispg, &floatmin, &floatmax);
    fprintf(stderr, "min(disp) = %g; max(disp) = %g\n", floatmin,
	    floatmax);
    free(dispf);
    WritePGMImage("disp.pgm", cols1, rows1, 255, dispg);
    free(dispg);
    floatmin = floatmax = 0.0;
    image_float_to_gray(scoref, cols1, rows1, scoreg, &floatmin, &floatmax);
    fprintf(stderr, "min(score) = %g; max(score) = %g\n", floatmin,
	    floatmax);
    free(scoref);
    WritePGMImage("score.pgm", cols1, rows1, 255, scoreg);
    free(scoreg);
#endif
    free(image1f);
    free(image2f);
    exit(0);
}

/* soustraction */
void
soustraireImage(const float *image1data, int dimx, int dimy,
		const float *image2data, float *resultatdata)
{
    /* taille de l'image */
    const int max_xy = dimx * dimy;
    int xy;

    for (xy = 0; xy < max_xy; xy++) {
	resultatdata[xy] = image1data[xy] - image2data[xy];
    }
}
